# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from .. import this

import matplotlib

if this.backend == 'PyQt4':
    PyQtAgg_str = 'Qt4Agg'
    matplotlib.use(PyQtAgg_str)
    matplotlib.rcParams['backend.qt4'] = this.backend
    import matplotlib.backends.backend_qt4agg as backend_qtagg
elif this.backend == 'PySide':
    PyQtAgg_str = this.backend
    matplotlib.use('Qt4Agg')
    matplotlib.rcParams['backend.qt4'] = this.backend
    import matplotlib.backends.backend_qt4agg as backend_qtagg
elif this.backend == 'PyQt5':
    PyQtAgg_str = 'Qt5Agg'
    matplotlib.use(PyQtAgg_str)
    matplotlib.rcParams['backend.qt5'] = this.backend
    import matplotlib.backends.backend_qt5agg as backend_qtagg
else:
    raise NotImplementedError

def _matplotlib_use(*pargs, **cargs):
    print('Warning: matplotlib.use was called by SiQt for the {} backend.\n'.format(this.backend) +\
     '       Calling it a second time will have no effect!')

matplotlib.use = _matplotlib_use

class NavigationToolbar(backend_qtagg.NavigationToolbar2QT):
    # only display the buttons we need
    toolitems = [t for t in backend_qtagg.NavigationToolbar2QT.toolitems if
                 t[0] in ('Home', 'Pan', 'Zoom', 'Save')]



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


import sys
import os
import re

def _resource_path(relative):
    """ Normalisation of the ressource path for PyInstaller """
    if hasattr(sys, "_MEIPASS"):
        return os.path.join(sys._MEIPASS, relative)
    else:
        basedir = os.path.split(os.path.split(__file__)[0])[0]
        return os.path.join(basedir, relative)





# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


# adapted from https://stackoverflow.com/questions/2467590/dynamically-evaluating-simple-boolean-logic-in-python

def create_token_lst(s, str_to_token):
    """create token list:
    'True or False' -> [True, lambda..., False]"""
    s = s.replace('(', ' ( ')
    s = s.replace(')', ' ) ')
    out = []
    for it in s.split():
        if it in ['not', 'and', 'or']:
            out.append(it)
        else:
            out.append(str_to_token[it])
    return out


def find(lst, what, start=0):
    return [i for i,it in enumerate(lst) if it == what and i >= start]


def parens(token_lst):
    """returns:
        (bool)parens_exist, left_paren_pos, right_paren_pos
    """
    left_lst = find(token_lst, '(')

    if not left_lst:
        return False, -1, -1

    left = left_lst[-1]

    #can not occur earlier, hence there are args and op.
    right = find(token_lst, ')', left + 3)[0]

    return True, left, right


def binary_eval(op, left, right):
    """token_lst has length 3 and format: [left_arg, operator, right_arg]
    operator(left_arg, right_arg) is returned"""
    return op(left, right)

def unary_eval(op, right):
    """token_lst has length 3 and format: [left_arg, operator, right_arg]
    operator(left_arg, right_arg) is returned"""
    return op(right)

def generic_eval(tokens, op_dict):
    if len(tokens) == 0:
        return True
    elif len(tokens) == 1:
        return tokens[0]
    elif len(tokens) == 2:
        return op_dict[tokens[0]](tokens[1]) # unary_eval
    elif len(tokens) == 3:
        try:
            return op_dict[tokens[1]](tokens[0], tokens[2]) # binary_eval
        except:
            print(tokens)
            raise
    elif len(tokens) > 3:
        if len(tokens) % 2 != (1 + tokens.count('not')) % 2:
            raise ValueError('Parsing failed, wrong lenght {} for {} with {} "not"'.format(
                len(tokens), tokens, tokens.count('not')))
        for key in ['not', 'and', 'or']:
            res = find(tokens, key)
            if res:
                k = res[0]
                if key == 'not':
                    mask = slice(k, k+2)
                else:
                    mask = slice(k-1, k+2)
                tokens[mask] = [generic_eval(tokens[mask], op_dict)]
                break
        else:
            raise ValueError("None of 'not', 'and', 'or' found in {}".format(tokens))
        return generic_eval(tokens, op_dict)

def formatted_bool_eval(token_lst, op_dict):
    """eval a formatted (i.e. of the form 'ToFa(ToF)') string"""
    if not token_lst:
        return None

    if len(token_lst) == 1:
        return token_lst[0]

    has_parens, l_paren, r_paren = parens(token_lst)

    if not has_parens:
        return generic_eval(token_lst, op_dict)

    token_lst[l_paren:r_paren+1] = [generic_eval(token_lst[l_paren+1:r_paren], op_dict)]

    return formatted_bool_eval(token_lst, op_dict)


class LogicParser(object):
    def __init__(self, tokens={}):
        self.default_tokens = {'True': True,
                            'False': False,
                            'and': lambda left, right: left and right,
                            'or': lambda left, right: left or right,
                            'not': lambda right: not right,
                            '(':'(',
                            ')':')'}
        self.tokens = tokens

    def __call__(self, expr):
        """The actual 'eval' routine,
        if 'expr' is empty, 'True' is returned,
        otherwise 'expr' is evaluated according to parentheses nesting.
        The format assumed:
            [1] 'LEFT OPERATOR RIGHT',
            where LEFT and RIGHT are either:
                    True or False or '(' [1] ')' (subexpression in parentheses)
        """

        tokens = dict(self.default_tokens)
        tokens.update(self.tokens)

        token_lst = create_token_lst(expr, str_to_token=tokens)
        return formatted_bool_eval(token_lst, op_dict=self.default_tokens)




# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
#from __future__ import unicode_literals

import os

valid_backends = ['PyQt4', 'PyQt5', 'PySide']

# need to implement https://www.python.org/dev/peps/pep-0302/


def _try_import(backend):
    try:
        __import__(backend)
        return True
    except ImportError:
        return False

for backend in valid_backends:
    if _try_import(backend):
        SIQT_BACKEND = backend
        break
else:
    raise ValueError('None of the {} seems to be installed!'.format(valid_backends)) 

if 'SIQT_BACKEND' in os.environ:
    SIQT_BACKEND = os.environ['SIQT_BACKEND']
    SIQT_BACKEND = _normalise_name(SIQT_BACKEND)

#if SIQT_BACKEND == 'PyQt4':
#    import PyQt4 as PyQt
#    from PyQt4 import QtCore
#    from PyQt4 import QtGui
#    from PyQt4 import Qt
#    QtPrintSupport = QtGui
#    QtWidgets = QtGui
#    QtCore.Signal = QtCore.pyqtSignal
#    QtCore.Slot = QtCore.pyqtSlot
#elif SIQT_BACKEND == 'PySide':
#    import PySide as PyQt
#    from PySide import QtCore
#    from PySide import QtGui
#    #from PySide import Qt
#    QtPrintSupport = QtGui
#    QtWidgets = QtGui
#    QtCore.pyqtSignal = QtCore.Signal
#    QtCore.pyqtSlot = QtCore.Slot
#    Qt = QtCore.Qt
#elif SIQT_BACKEND == 'PyQt5':
#    import PyQt5 as PyQt
#    from PyQt5 import Qt
#    from PyQt5 import QtCore
#    from PyQt5 import QtGui
#    from PyQt5 import QtPrintSupport
#    from PyQt5 import QtWidgets
#    QtCore.Signal = QtCore.pyqtSignal
#    QtCore.Slot = QtCore.pyqtSlot
#else:
#    raise NotImplementedError('Backend {} is not supported!'.format(SIQT_BACKEND))




# -*- coding: utf-8 -*-

import sys

from . import siqt
from ._version import __version__


__version_date__ = "Sun Feb 14 14:28:51 2016 +0100"
__version_hash__ = "1d5f7f3"


this = sys.modules[__name__] 
this.backend = None

use = siqt.importers.use



# -*- coding: utf-8 -*-

import imp
import sys

name = "QtCore"
path = ['/usr/lib64/python2.7/site-packages/PyQt4']


def load_module(name, path):
    import sip
    import importlib
    print(name, path)
    fp, pathname, description = imp.find_module(name, path)
    print(sys.modules.keys())

    try:
        module = imp.load_module(name, fp, pathname, description)
    except:
        module = importlib.import_module('PyQt4.QtCore') #__import__('PyQt4.QtCore', globals(), locals(), [], -1)
    finally:
        if fp:
            fp.close()
    return module

mod = load_module(name, path)
print(mod.QT_VERSION_STR)






# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from collections import OrderedDict

import six

from .qtbase import QtCore
from .qtbase import QtGui
from .qtbase import QtWidgets

from .dep_resolv import dependency_graph, calculate_dependencies

try:
    from .qtbase.QtCore import QString
except ImportError:
    QString = str



# The following functions are helper functions to setup the GUI
def add_actions(self, target, actions):
    for action in actions:
        if action is None:
            target.addSeparator()
        else:
            target.addAction(action)


def create_action(self, text, slot=None, shortcut=None, 
                    icon=None, tip=None, checkable=False, 
                    signal="triggered"):
    action = QtWidgets.QAction(text, self)
    if icon is not None:
        action.setIcon(QtWidgets.QIcon(":/%s.png" % icon))
    if shortcut is not None:
        action.setShortcut(shortcut)
    if tip is not None:
        action.setToolTip(tip)
        action.setStatusTip(tip)
    if slot is not None:
        getattr(action, signal).connect(slot)
    if checkable:
        action.setCheckable(True)
    return action


def menu_generator(self, name, label, elements):
    # File menu
    self.menu[name] = self.menuBar().addMenu("&"+label)
    self.menu[name].elmts = elements

    def show_menu(action):
        def f(value):
            if value:
                action.setDisabled(False)
                action.setEnabled(True)
            else:
                action.setDisabled(True)
                action.setEnabled(False)

        return f

    idx = 1
    for key, menu_el  in elements.items():
        if menu_el is None:
            self.menu[name].addSeparator()
            continue


        pars = self.menu[name].elmts[key]

        if name == 'view':
            pars['slot'] = self.on_view_handler(key)
            if idx < 10:
                pars['shortcut'] = 'Ctrl+{}'.format(idx)

        if 'slot' not in pars:
            pars['slot'] = getattr(self, 'on_'+key)

        tpars = pars.copy()
        for tkey in ['depends', 'enabled']:
            if tkey in tpars: 
                del tpars[tkey]
        action = self.create_action(**tpars)
        if pars['depends']:  # has some dependecies meaning can't be used initially
            action.setDisabled(True)

        if 'enabled' in pars and not pars['enabled']: 
            action.setDisabled(True)
        self.menu[name].addAction(action)

        pars['qtobject'] = action
        pars['show'] = show_menu(action)
        idx += 1


def set_dep_flag_recursive(self, key, value, sync=False):
    """
    Change status for a dependent field, and pull all the fields that are dependent on it.
    """
    self.dep_flags[key] = value
    for dkey in dependency_graph(key, self.dep_graph):
        self.dep_flags[dkey] = False
    if sync:
        calculate_dependencies(self)


class SiqtElement(dict):

    def __init__(self, qtobj, layout=None, position=None, depends=[], dtype=str, **args):
        """
        A helper class to work with the underlying QtObject
        """
        if isinstance(qtobj, six.string_types):
            qtobj = QtWidgets.QLabel(qtobj)  # if given a string, this is probably a label
        self.dtype = dtype
        if isinstance(qtobj, QtWidgets.QLineEdit) and dtype != str:
            if dtype in [float]: #, np.float]:
                validator = QtGui.QDoubleValidator()
                qtobj.setValidator(validator)
                args['validator'] = validator
            elif dtype in [int]: #, np.int]:
                validator = QtGui.QIntValidator()
                qtobj.setValidator(validator)
                args['validator'] = validator
        if isinstance(qtobj, QtWidgets.QComboBox) and 'choices' in args:
            args['choices'] = OrderedDict(args['choices'])
            for key, val in args['choices'].items(): 
                qtobj.addItem(QString(key), val)

        super(SiqtElement, self).__init__(qtobj=qtobj, depends=depends, **args)
        if layout is not None and position is not None:
            layout.addWidget(qtobj, *position)

    def set_text(self, value):
        value = QString(str(value))
        self['qtobj'].setText(value)

    def set_choices(self, choices):
        control  = self['qtobj']
        if not isinstance(control, (QtWidgets.QComboBox, QtWidgets.QListWidget)):
            raise NotImplementedError('set_choices is only valid for a QComboBox element')

        control.blockSignals(True)
        if isinstance(control, QtWidgets.QComboBox):
            choices = OrderedDict(choices)
            for idx in range(control.count()):
                control.removeItem(0)

            for key, val in choices.items():
                control.addItem(key, val)
        elif isinstance(control, QtWidgets.QListWidget):
            control.clear()
            for key in sorted(choices):
                control.addItem(key)

        self['choices'] = choices
        control.blockSignals(False)

    def set_value(self, value):
        qtobj = self['qtobj']
        if isinstance(qtobj, QtWidgets.QCheckBox):
            if value:
                value =  QtCore.Qt.Checked
            else:
                value = QtCore.Qt.Unchecked
            qtobj.setCheckState(value)
        elif isinstance(qtobj, (QtWidgets.QComboBox, QtWidgets.QListWidget)):
            self.set_choices(value)
        elif isinstance(qtobj, QtWidgets.QSlider):
            raise NotImplementedError
        else:
            qtobj.setText(str(value))



    @property
    def value(self):
        qtobj = self['qtobj']
        if isinstance(qtobj, QtWidgets.QCheckBox):
            return qtobj.isChecked()
        elif isinstance(qtobj, QtWidgets.QComboBox):
            key = str(qtobj.currentText())
            return self['choices'][key]
        elif isinstance(qtobj, QtWidgets.QSlider):
            return self.dtype(qtobj.value())
        elif isinstance(qtobj, QtWidgets.QListWidget):
            return [str(el.text()) for el in qtobj.selectedItems()]
        else:
            val = str(qtobj.text())
            if not val and self.dtype != str:
                val = 0
            return self.dtype(val)

    def __getattr__(self, name):
        """ Only called if name not in the registered methods
        In this case call the qtobject methods """

        return getattr(self['qtobj'],  name)









#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from .debug_information import DebugInfoWidget
from .progress_bar import ProgressBarWidget





#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


from ..qtbase import QtCore
from ..qtbase import QtGui
from ..qtbase import QtWidgets


from ..dep_resolv import calculate_dependencies

class ProgressBarWidget(QtWidgets.QWidget):

    def __init__(self, main_window, parent = None):
        super(ProgressBarWidget, self).__init__(parent)
        layout = QtGui.QVBoxLayout(self)
        self.setGeometry(QtCore.QRect(100, 100, 400, 200))

        # Create a progress bar and a button and add them to the main layout
        self.progressBar = QtGui.QProgressBar(self)
        self.progressBar.setRange(0,100)
        layout.addWidget(self.progressBar)

        self.myLongTask = TaskThread(parent, main_window)
        self.progressTask = ProgressThread(parent, main_window)
        #self.progressBar.setRange(0,0)
        self.myLongTask.start()
        self.progressTask.start()
        self.myLongTask.taskFinished.connect(self.onFinished)

        self.progressTask.notifyProgress.connect(self.onProgress)
        self.base = main_window


    def onProgress(self, i):
        self.progressBar.setValue(i)


    def onFinished(self):
        self.base.tabs['convolution']['threshold_slider']['qtobj'].setRange(0, self.base.gpr.cxx_max.max())

        self.base.view_mode = 'convolution'
        self.base.on_draw()
        self.base.set_dep_flag_recursive('convolution', True)

        calculate_dependencies(self.base)

        self.progressTask.terminate()
        self.close()


class TaskThread(QtCore.QThread):
    taskFinished = QtCore.pyqtSignal()

    def __init__(self, parent, main_window):
        QtCore.QThread.__init__(self, parent)
        self.base = main_window

    def run(self):
        self.base.gpr.convolve()
        self.taskFinished.emit()  


class ProgressThread(QtCore.QThread):
    notifyProgress = QtCore.pyqtSignal(int)

    def __init__(self, parent, main_window):
        QtCore.QThread.__init__(self, parent)
        self.base = main_window

    def run(self):
        from time import sleep
        res = 0
        self.base.gpr.conv_progress = 0
        while res<98:
            res = int(self.base.gpr.conv_progress)
            self.notifyProgress.emit(res)
            sleep(0.5)



# -*- coding: utf-8 -*-

import SiQt
SiQt.use('PyQt5', force=True)

import PyQt4
from PyQt4 import QtCore

print('PyQt4.__name__              :', PyQt4.__name__)
print('SiQt.backend                :', SiQt.backend)
print('PyQt4.QtCore.QT_VERSION_STR :', QtCore.QT_VERSION_STR)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from SiQt.siqt.dep_resolv import check_depflags

def assert_equal(a, b):
    assert a==b


def test_check_depflags():
    dep_flags = {'a': True, 'b': False, 'c': True, 'd': False}
    yield assert_equal, check_depflags(dep_flags, ['a', 'c']), True
    yield assert_equal, check_depflags(dep_flags, ['a', 'd']), False
    yield assert_equal, check_depflags(dep_flags, ['a', 'b', 'd']), False
    yield assert_equal, check_depflags(dep_flags, 'a'), True
    yield assert_equal, check_depflags(dep_flags, 'a and c'), True
    yield assert_equal, check_depflags(dep_flags, 'a and d'), False
    yield assert_equal, check_depflags(dep_flags, 'a and c and d'), False
    yield assert_equal, check_depflags(dep_flags, 'a or d'), True
    yield assert_equal, check_depflags(dep_flags, 'c and (a or d)'), True
    yield assert_equal, check_depflags(dep_flags, 'c and not d'), True
    yield assert_equal, check_depflags(dep_flags, '(c or d) and (not d or b)'), True










# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import functools
from unittest.case import SkipTest
from multiprocessing import Process, Queue
from time import sleep, time


def _run_process(func):
    @functools.wraps(func)
    def inner_wrapper(*args):
        def func_wrapper(func, q, args):
            try:
                res = func(*args)
                q.put(res)
            except Exception as e:
                q.put(e)

        q = Queue()
        p = Process(target=func_wrapper, args=(func, q, args))
        p.start()

        result = q.get(block=True)
        if isinstance(result, Exception):
            raise result

        return result
    return inner_wrapper


@_run_process
def _check_importer(backend):
    import SiQt
    try:
        import SiQt.QtCore
    except ImportError:
        assert True  # this is normal
    except:
        raise
    try:
        SiQt.use(backend, force=False)
    except ImportError:
        raise SkipTest
    except:
        raise
    import SiQt.QtCore


def test_PyQt4_importer():
    _check_importer('PyQt4')

def test_PyQt5_importer():
    _check_importer('PyQt5')

def test_PySide_importer():
    _check_importer('PySide')



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from ..qtbase import QtCore
from ..qtbase import QtWidgets

from ..tests import get_system_info

class DebugInfoWidget(QtWidgets.QTreeWidget):

    def __init__(self, main_window, parent = None):
        super(DebugInfoWidget, self).__init__(parent)
        self.setWindowTitle('Debug information')
        self.setGeometry(QtCore.QRect(100, 100, 500, 800))

        dinfo = get_system_info(pretty_print=False)

        header = QtWidgets.QTreeWidgetItem(["Parameter", "Value"])
        self.setHeaderItem(header) 

        for sec_key, sec_value in dinfo.items():
            header = QtWidgets.QTreeWidgetItem(self, [sec_key])
            header.setExpanded(True)

            for key in sorted(sec_value):
                val = sec_value[key]
                label = str(val)
                QtWidgets.QTreeWidgetItem(header, [key, label])

            self.base = main_window
            self.header().resizeSection(0, 230)


    def closeEvent(self, event):
        self.base.widgets['info'] = None
        super(DebugInfoWidget, self).closeEvent(event)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


from .siqt.importers import use

import os

pars = {'mode': 'compatible'}
if 'SIQT_BACKEND_FORCE' in os.environ:
    pars['backend_name'] = os.environ['SIQT_BACKEND_FORCE']
    pars['force'] = True
elif 'SIQT_BACKEND' in os.environ:
    pars['backend_name'] = os.environ['SIQT_BACKEND']
    pars['force'] = False
else:
    raise ValueError('Environemental variables SIQT_BACKEND or SIQT_BACKEND_FORCE not found!')

if 'SIQT_MODE' in os.environ:
    pars['mode'] = os.environ['SIQT_MODE']

use(**pars)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function


import six


def show_qt_control_element(el):
    def f(status=False):
        #el.setDisabled(not status)
        el.setEnabled(status)
    return f


def sync_gui(lock=[], update=[], view_mode=None, background=False):
    from .qtbase import QtWidgets
    def decorator_generator(f):
        def f_wrapper(self, *args, **kwargs):
            # lock some elements if need be
            for el in lock:
                p = el(self)
                if 'show' in p:
                    p['show'](False)
                    p['is_visible'] = False
                else:
                    # go one level down, this is a dict of dics
                    for key in p:
                        p[key]['show'](False)
                        p[key]['is_visible'] = False


            for key in update:
                self.set_dep_flag_recursive(key, True)

            QtWidgets.QApplication.processEvents()

            f(self, *args, **kwargs)

            if view_mode is not None:
                self.view_mode = view_mode
                self.on_draw()
            if not background:
                calculate_dependencies(self, verbose=False)
        return f_wrapper
    return decorator_generator

def dependency_graph(key, storage):
    out = []
    if key in storage:
        fields = storage[key]
        out += fields
        for pkey in fields:
            new_fields = dependency_graph(pkey, storage)
            if new_fields:
                out += new_fields
    return out


def check_depflags(dep_flags, expr):
    """ Check if the boolean expression expr is verified in dep_flags
    Two possibilities for expr:
      ['flag1', 'flag2', etc]
    a boolean expression
      'flag1 and (flag2 or not flag3)'

    """
    if isinstance(expr, (list, tuple)):
        res = [dep_flags[dkey] for dkey in expr]
        if not len(res):
            return True
        else:
            return all(res)
    elif isinstance(expr, six.string_types):
        from .logic_parser import LogicParser
        lp = LogicParser(dep_flags)
        return lp(expr)

    else:
        raise NotImplementedError




def calculate_dependencies(self, verbose=False, initialize=False):
    """
        This function allows to determine which graphical should be active
        depending on the current processing step
    """
    dep_list = []

    for menu_el in self.menu.values():
        for key, el in  menu_el.elmts.items():
            if el is None:
                continue
            if 'enabled' in el and not el['enabled']:
                continue
            dep_list.append(('menu', key, el))
    for key, el in self.controls.items():
        dep_list.append(('controls', key, el))

    for tab_name, tab_el in self.tabs.items():
        for key, el in  tab_el.items():
            dep_list.append(('tabs', key, el))


    if initialize:
        for tab_name, tab_el in self.tabs.items():
            for key, el in  tab_el.items():
                if 'show' not in el:
                    el['show'] = show_qt_control_element(el['qtobj'])
        for mtype, key, el in dep_list:
            if 'is_visible' not in el:
                el['is_visible'] = None

    for mtype, key, el in dep_list:
        # check if the dependency for this control element is verified
        res = check_depflags(self.dep_flags, el['depends'])
        if isinstance(res, bool):
            pass
        elif not len(res):
            res = True
        else:
            res = all(res)
        if verbose:
            print('{:10} {:20} {:10} {:10}'.format(mtype, key, el['is_visible'], str(res)))

        if el['is_visible'] != res:
            try:
                el['show'](res)
            except:
                print(el)
                raise
            el['is_visible'] = res



# -*- coding: utf-8 -*-

import os
import sys

_base_dir, _ = os.path.split(__file__)

def run(coverage=True):
    import nose
    argv=['', '-s', '--where={}'.format(_base_dir), '--verbosity=2']
    if coverage:
        argv += ['--with-coverage', '--cover-package=SiQt']
    result = nose.run(argv=argv)
    status = int(not result)
    return status

def run_cli(coverage=False):
    status = run(coverage=coverage)
    print('Exit status: {}'.format(status))
    sys.exit(status)



if __name__ == '__main__':
    run_cli()





#!/usr/bin/python
# -*- coding: utf-8 -*-

# to upload to PyPi, see
# https://stackoverflow.com/questions/14219499/pypi-server-response-500 
#from distutils.core import setup
from setuptools import setup, find_packages
import os
import sys
import re


# a define the version sting inside the package
# see https://stackoverflow.com/questions/458550/standard-way-to-embed-version-into-python-package
VERSIONFILE="SiQt/_version.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^__version__ = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    version = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

with open('README.rst', 'r') as fh:
    long_description = fh.read()


setup(name='SiQt',
      version=version,
      description='A compatibility library for PyQt4, PyQt5 and PySide',
      author='Roman Yurchak',
      author_email='rth.yurchak@gmail.com',
      url='https://github.com/rth/SiQt',
      license='MIT',
      packages=find_packages(),
      install_requires=['six'],
      long_description=long_description,
      classifiers=[
      # How mature is this project? Common values are
      #   3 - Alpha
      #   4 - Beta
      #   5 - Production/Stable
      #'Development Status :: 2 - Pre-Alpha',

      # Indicate who your project is intended for
      'Intended Audience :: Developers',
      'Topic :: Software Development :: Libraries :: Python Modules',

      # Pick your license as you wish (should match "license" above)
      'License :: OSI Approved :: MIT License',

      # Specify the Python versions you support here. In particular, ensure
      # that you indicate whether you support Python 2, Python 3 or both.
      'Programming Language :: Python :: 2',
      'Programming Language :: Python :: 2.7',
      'Programming Language :: Python :: 3',
      'Programming Language :: Python :: 3.3',
      'Programming Language :: Python :: 3.4',
      'Programming Language :: Python :: 3.5',
      ],
     )




#!/usr/bin/python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

import sys
from .run_suite import run, run_cli

def get_system_info(pretty_print=False):
    import siqt
    from siqt import QtCore
    import platform
    system = {}
    for key in ['architecture', 'machine', 'platform', 'processor',
                'system']:
        system[key] = getattr(platform, key)()

    # platform specific stuff
    for key in ['win32_ver', 'mac_ver', 'linux_distribution']:
        res = getattr(platform, key)()
        if res[0]:
            system[key] = res
    if  sys.maxsize > 2**32:
        system['architecture_size'] = '64 bits'
    else:
        system['architecture_size'] = '32 bits'

    python = {}
    for key in [ 'python_implementation', 'python_version', 'python_compiler']:
        python[key] = getattr(platform, key)()

    sv = {}
    try:
        import numpy as np
        sv['numpy'] = np.__version__
    except ImportError:
        sv['numpy'] = 'Not installed'

    try:
        import scipy
        sv['scipy'] = scipy.__version__
    except ImportError:
        sv['scipy'] = "Not installed"

    try:
        import pandas as pd
        sv['pandas'] = pd.__version__
    except ImportError:
        sv['pandas'] = "Not installed"
    try:
        import matplotlib as mpl
        sv['matplotlib'] = mpl.__version__
    except ImportError:
        sv['matplotlib'] = "Not installed"
    if 'PyQt' in siqt.SIQT_BACKEND:
        from siqt import Qt
        sv['Qt'] = QtCore.QT_VERSION_STR
        sv['PyQt'] = Qt.PYQT_VERSION_STR
    elif 'PySide' in siqt.SIQT_BACKEND:
        import PySide
        sv['Qt'] = QtCore.__version__
        sv['PySide'] = PySide.__version__
    else:
        raise NotImplementedError



    if not pretty_print:
        return {'system': system, 'python': python, 'software_versions': sv}
    else:
        FMT = '   {:12}: {}'
        s = []
        for label, mdict in [('System information', system),
                             ('Python version', python),
                             ('Included modules', sv)]:
            s += ['', label, '='*30, '']
            for key, val in mdict.items():
                if type(val) is tuple:
                    val = ' '.join(val)
                s.append(FMT.format(key, val))
        s.append('')
        

        return '\n'.join(s)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import sys
import os

import pkgutil
import os.path
import six


valid_backends = ['PyQt4', 'PyQt5', 'PySide']

def _normalise_name(name):
    for backend in valid_backends:
        if name.lower() == backend.lower():
            return backend
    else:
        raise ValueError('SiQt backend {} is not supported!\n Must be one of {}'.format(
                            name, valid_backends))

def use(backend_name, force=False, mode='strict', matplotlib_hook=False):
    """
    Initialize a SiQt backend.

    Parameters
    ----------
     - backend_name: backend name. One of ['PyQt4', 'PyQt5', 'PySide']
     - force: use import hooks to overwrite direct import of the above
       packages by SiQt
     - mode: the import hooks to use
            * "strict": use the backend as it is
            * "compatible": monkeypatch the backend to smooth the differences bewteen
                        backends (e.g. will add SiQt.QtWidgets with the PyQt4 backend,
                        which was only added in PyQt5)
            * "porting": same as smooth, but prints a warning every time an import hook is used.
     - matplotlib_hook: call matplotlib.use with the correct backend
            when matplotlib is first imported
    """
    import sys
    from .. import this
    #from future.standard_library import RenameImport
    name = _normalise_name(backend_name)
    pkg = pkgutil.find_loader(name)
    if pkg is None:
        raise ImportError('Backend {} does not seem to be installed and cannot be used!')
    if this.backend is not None:
        if this.backend != name:
            print('Warning: SiQt.use should be called before any imports of PyQt/PySide.\n'\
              '         Changing backend on the runtime (i.e. calling it a second time) is not supported!')
        return
    this.backend = name
    if six.PY3:
        path_new = os.path.dirname(pkg.path)
    else:
        path_new = pkg.filename
    this.__path__.append(path_new)

    sys.meta_path.insert(0, RenameImportFinder())
    if six.PY2:
        if this.backend in ['PyQt4', 'PyQt5']:
            import sip # just initializing
    if matplotlib_hook:
        sys.meta_path.insert(0, MatplotlibImporter())


class MatplotlibImporter(object):
    def __init__(self):
        pass

    def find_module(self, fullname, path=None):
        import sys
        if fullname.startswith('matplotlib'):
            sys.meta_path = sys.meta_path[1:] # remove the matplotlib hook
            # now initialize matplotlib with the correct Qt4 backend
            from . import matplotlib
        return None

    def load_module(self, name):
        """This is never going to be called"""
        pass

class RenameImportFinder(object):
    """
    Adapted from http://python-future.org/_modules/future/standard_library.html
    """
    def __init__(self):
        pass

    def find_module(self, fullname, path=None):
        # Handles hierarchical importing: package.module.module2
        for name in valid_backends:
            if fullname.startswith(name):
                print('Finding', fullname, path)
                return RenameImportLoader(fullname, path)
        return None

class RenameImportLoader(object):
    """
    Adapted from http://python-future.org/_modules/future/standard_library.html
    """

    def __init__(self, name, path=None):
        '''
        Pass in a dictionary-like object mapping from old names to new
        names. E.g. {'ConfigParser': 'configparser', 'cPickle': 'pickle'}
        '''
        self.name_orig = name
        self.path = path

    def load_module(self, name, path=None):
        if path is None:
            path = self.path
        #print('Loading', name, path)
        for backend_name in valid_backends:
            if backend_name in name:
                name = name.replace(backend_name, 'SiQt')
                self.name_new = name
                break
        #print('2. Loading', name)
        if name in sys.modules:
            #print('zzzz', name)
            #print(sys.modules.keys())
            module = sys.modules[name]
            if self.name_orig not in sys.modules:
                sys.modules[self.name_orig] = module
        else:
            #print('aaa')
            module = self._find_and_load_module(name, path)
            sys.modules[name] = module
            sys.modules[self.name_orig] = module
        #module.__spec__.name = self.name_orig
        #print(module.__spec__)
        return module


    def _find_and_load_module(self, name, path=None):
        """
        Finds and loads it. But if there's a . in the name, handles it
        properly.
        """
        print('3.', name, path)
        import imp
        bits = name.split('.')
        while len(bits) > 1:
            # Treat the first bit as a package
            packagename = bits.pop(0)
            #print(packagename)
            package = self._find_and_load_module(packagename, path)
            try:
                path = package.__path__
            except AttributeError:
                # This could be e.g. moves.
                #flog.debug('Package {0} has no __path__.'.format(package))
                if name in sys.modules:
                    return sys.modules[name]
                #flog.debug('What to do here?')

        #if six.PY2:
        name = bits[0]
        if name in sys.modules:
            #print('zzzz', name, six.PY2)
            #sys.modules[self.name_orig] = name
            module = sys.modules[name]
            #print(module.__spec__)
        else:
            #print('4.', name, path)
            fp, pathname, description = imp.find_module(name, path)
            print(name, self.name_new,  fp, pathname, description)
            try:
                #print(self.name_orig)
                module = imp.load_module(self.name_orig, fp, pathname, description)
            except Exception as e:
                #if six.PY2:
                #    import importlib
                #    module = importlib.import_module(self.name_orig) 
                #else:
                raise e
            finally:
                if fp:
                    fp.close()
        return module
        #else: # PY3
        #    import importlib.machinery
        #    print('Machinery')
        #    module = importlib.machinery.SourceFileLoader(name, path).load_module()
        #    return module




## some earlier attemps should be removed
#import sys
#
#def name(item):
#    " Return an item's name. "
#    return item.__name__
#    
#def format_arg_value(arg_val):
#    """ Return a string representing a (name, value) pair.
#    
#    >>> format_arg_value(('x', (1, 2, 3)))
#    'x=(1, 2, 3)'
#    """
#    arg, val = arg_val
#    return "%s=%r" % (arg, val)
#
#
#def echo(fn):
#    """ Echo calls to a function.
#    
#    Returns a decorated version of the input function which "echoes" calls
#    made to it by writing out the function's name and the arguments it was
#    called with.
#    """
#    import functools
#    # Unpack function's arg count, arg names, arg defaults
#    @functools.wraps(fn)
#    def wrapped(*v):
#        # Collect function arguments by chaining together positional,
#        # defaulted, extra positional and keyword arguments.
#        v = list(v)
#        name = v[0]
#        if name in valid_backends:
#            v[0] = 'SiQt'
#        elif name == 'SiQt':
#            pass
#        elif hasattr(name, '__spec__'):
#            print(dir(name))
#            spec = name.__spec__
#            spec.name = 'SiQt'
#            name.__name__ = 'SiQt'
#        else:
#            print(name)
#
#        print("%s %s" % (fn, v))
#        res = fn(*v)
#        print("Out: %s" % res)
#        return res
#    return wrapped
#
#
#class HijackPyQtImport(object):
#
#    def find_module(self, fullname, path=None):
#        import sys
#        from .. import this
#        for backend_name in valid_backends:
#            if fullname.startswith(backend_name):
#                fullname_out = fullname.replace(backend_name, 'SiQt')
#                print(backend_name, fullname,  path, fullname_out)
#                print(sys.meta_path)
#                if six.PY3:
#                    import importlib.machinery
#                    print(fullname_out, path)
#                    loader =  importlib.machinery.SourceFileLoader(fullname_out, path)
#                    for attr in dir(loader):
#                        if not attr.startswith('__'):
#                            setattr(loader, attr, echo(getattr(loader, attr)))
#                    return loader
#                    #return self._py3_loader(fullname_out, path)
#                else:
#                    raise NotImplementedError
#        else:
#            return None
